import torch
from torchvision import datasets, transforms
from torch.utils.data import DataLoader
from torchvision.utils import make_grid
from utils import show
import numpy as np
import os


class SparseAutoencoder(torch.nn.Module):

    def __init__(self, inputDims, hDims):
        super().__init__()
        self.inputDims = inputDims
        self.hDims = hDims

    def init_weights(self, layer):
        torch.nn.init.xavier_uniform_(layer.weight)
        torch.nn.init.constant_(layer.bias, 0)

    def forward(self, x):
        """ Return both the representation and the reconstructed image """
        raise NotImplementedError

    def batched_sparsity_penalty(self, encoded):
        raise NotImplementedError

    def loss_function(self, x_hat, x, encoded):
        raise NotImplementedError

if __name__ == "__main__":

    # This follows standard practice.
    # The combo of the to tensor and the normalize effectively normalizes
    # the pixel values to have zero mean and unit standard deviation.
    transform = transforms.Compose([
        transforms.ToTensor(),
        transforms.Normalize((0.1307,), (0.3081,))
    ])

    train_dataset = datasets.MNIST(
        root='./data',
        train=True,
        transform=transform,
        download=True
    )

    train_dataloader = DataLoader(
        dataset=train_dataset,
        batch_size=64,
        shuffle=True
    )

